#include <Wire.h>

int64_t C[7];
unsigned long timeConversion; // us

void MS5611Transmit(byte address, int command) {
  Wire.beginTransmission(address);
  Wire.write(command);
  Wire.endTransmission();
}

int64_t MS5611Receive(byte address, int command, int bytes) {
  MS5611Transmit(address, command);
  Wire.requestFrom(address, bytes);
  int64_t R = 0;
  for (int i = 0; i < bytes; i++) R = (R << 8) | Wire.read();
  return R;
}

void MS5611Reset(byte address) {
  MS5611Transmit(address, 0x1e);
  delayMicroseconds(3000);
}

void MS5611ConvertD1(byte address) { // pressure conversion
  MS5611Transmit(address, 0x48); // OSR=4096
  timeConversion = micros() + 9040;
}

void MS5611ConvertD2(byte address) { // temperature conversion
  MS5611Transmit(address, 0x50); // OSR=256
  timeConversion = micros() + 600;
}

int64_t MS5611ReadADC(byte address) {
  while (long(timeConversion - micros()) > 0); // wait for conversion to be done
  return MS5611Receive(address, 0x00, 3);
}

int64_t MS5611ReadCalibration(byte address, int index) {
  return MS5611Receive(address, 0xa0 + index * 2, 2);
}

void MS5611GetTemperaturePressure(byte address, float &temperature, float &pressure) {
  if (C[0] == 0) { // if not done yet initalize MS5611
    Wire.begin(); // start communication with MS5611 as master
    MS5611Reset(address); // reset MS5611
    for (int i = 1; i <= 6; i++) C[i] = MS5611ReadCalibration(address, i); // read calibration coefficients
    MS5611ConvertD1(address); // start the pressure conversion
    C[0] = 1; // set the initalized flag 
  }
  
  int64_t D1 = MS5611ReadADC(address); // read the converted pressure started last time
  MS5611ConvertD2(address); // start the temperature conversion
  int64_t D2 = MS5611ReadADC(address); // read the converted temperature
  MS5611ConvertD1(address); // start the pressure conversion to be ready to read next time
  
  int64_t dT = D2 - (C[5] << 8);
  int64_t TEMP = ((dT * C[6]) >> 23);
  int64_t OFF = (C[2] << 16) + ((C[4] * dT) >> 7);
  int64_t SENS = (C[1] << 15) + ((C[3] * dT) >> 8);
  if (TEMP < 0) {
    OFF -= (5 * TEMP * TEMP) >> 1;
    SENS -= (5 * TEMP * TEMP) >> 2;
    TEMP -= (dT * dT) >> 31;
  }
  temperature = 0.01 * (TEMP + 29315); // Kelvin
  pressure = (((D1 * SENS) >> 21) - OFF) >> 15; // Pa
}

