enum keys : byte {
  UP = 0xd0,
  DOWN = 0xb0,
  LEFT = 0x70,
  RIGHT = 0xe0
};

enum menus : byte {
  SETTINGS_SAVE,
  ALTITUDE_RESET,
  VARIO_SET_TEC,
  RX_VOLTAGE_1_CALIBRATE_VOLTAGE,
  RX_VOLTAGE_2_CALIBRATE_VOLTAGE,
  DRIVE_VOLTAGE_CALIBRATE_VOLTAGE,
  DRIVE_CURRENT_CALIBRATE_OFFSET,
  DRIVE_CURRENT_SET_SCALE,
  PITOT_PRESSURE_CALIBRATE_OFFSET,
  PITOT_PRESSURE_SET_SCALE,
  AIRSPEED_SET_RATIO,
  LAST = AIRSPEED_SET_RATIO
};

enum menuLevels : byte {
  MAIN,
  INFO,
  SET
};

int menu, menuLevel;
int setValue, setValueMin, setValueMax;
int setIncrement, setIncrementMax;
int setDecimalFactor;

void handleJetiBox() {
  static unsigned long timeKey;
  byte key = jetiEx.GetJetiboxKey();
  if (key == 0) return;
  if (long(timeKey - millis()) > 0) return;
  timeKey = millis() + 200;

  if (menuLevel == MAIN) {
    if (key == UP) {
      menu--;
      if (menu < 0) menu = LAST;
      setJetiboxText();
      return;
    }
    if (key == DOWN) {
      menu++;
      if (menu > LAST) menu = 0;
      setJetiboxText();
      return;
    }
    if (key == LEFT) {
      switch (menu) {
        case SETTINGS_SAVE:
          initEEPROM();
          break;
        case ALTITUDE_RESET:
          initAltitude();
          break;
        case VARIO_SET_TEC:
          initVarioTEC();
          break;
        case RX_VOLTAGE_1_CALIBRATE_VOLTAGE:
          initRxVoltage1Scale();
          break;
        case RX_VOLTAGE_2_CALIBRATE_VOLTAGE:
          initRxVoltage2Scale();
          break;
        case DRIVE_VOLTAGE_CALIBRATE_VOLTAGE:
          initDriveVoltageScale();
          break;
        case DRIVE_CURRENT_CALIBRATE_OFFSET:
          initDriveCurrentOffset();
          break;
        case DRIVE_CURRENT_SET_SCALE:
          initDriveCurrentScale();
          break;
        case PITOT_PRESSURE_CALIBRATE_OFFSET:
          initPitotPressureOffset();
          break;
        case PITOT_PRESSURE_SET_SCALE:
          initPitotPressureScale();
          break;
        case AIRSPEED_SET_RATIO:
          initAirspeedRatio();
          break;
      }
      jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Reset Done");
      return;
    }
    if (key == RIGHT) {
      switch (menu) {
        case SETTINGS_SAVE:
          EEPROM.put(0, set);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Saved");
          break;
        case ALTITUDE_RESET:
          initAltitude();
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Reset Done");
          break;
        case VARIO_SET_TEC:
          menuLevelSet(set.varioTEC, 0, 1, 1);
          break;
        case RX_VOLTAGE_1_CALIBRATE_VOLTAGE:
          menuLevelSet(rxVoltage1, 1, 9.999, 3);
          break;
        case RX_VOLTAGE_2_CALIBRATE_VOLTAGE:
          menuLevelSet(rxVoltage2, 1, 9.999, 3);
          break;
        case DRIVE_VOLTAGE_CALIBRATE_VOLTAGE:
          menuLevelSet(driveVoltage, 10, 99.99, 2);
          break;
        case DRIVE_CURRENT_CALIBRATE_OFFSET:
          menuLevel = INFO;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Disconnect Batt.");
          break;
        case DRIVE_CURRENT_SET_SCALE:
          menuLevelSet(set.driveCurrentScale * 1023 / VCC, 10, 99.99, 2);
          break;
        case PITOT_PRESSURE_CALIBRATE_OFFSET:
          menuLevel = INFO;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Cover Pitot Tube");
          break;
        case PITOT_PRESSURE_SET_SCALE:
          menuLevelSet(set.pitotPressureScale * 1023 / VCC, 100, 9999, 0);
          break;
        case AIRSPEED_SET_RATIO:
          menuLevelSet(set.airspeedRatio, 1.6, 2.5, 3);
          break;
      }
      return;
    }
    return;
  }
  
  if (menuLevel == SET) {
    if (key == UP) {
      setValue += setIncrement;
      if (setValue > setValueMax) setValue = setValueMax;
      setJetiboxText();
      return;
    }
    if (key == DOWN) {
      setValue -= setIncrement;
      if (setValue < setValueMin) setValue = setValueMin;
      setJetiboxText();
      return;
    }
    if (key == LEFT) {
      if (setIncrement < setIncrementMax) {
        setIncrement *= 10;
        setJetiboxText();
        return;
      }
      menuLevel = MAIN;
      setJetiboxText();
      return;
    }
    if (key == RIGHT) {
      if (setIncrement > 1) {
        setIncrement /= 10;
        setJetiboxText();
        return;
      }
      switch (menu) {
        case VARIO_SET_TEC:
          set.varioTEC = setValue > 0.5;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Done");
          break;
        case RX_VOLTAGE_1_CALIBRATE_VOLTAGE:
          set.rxVoltage1Scale = 0.001 * setValue / analogReadOversampling(RX_VOLTAGE_1_PIN, CALIBRATION_OVERSAMPLINGRATE);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibration Done");
          break;
        case RX_VOLTAGE_2_CALIBRATE_VOLTAGE:
          set.rxVoltage2Scale = 0.001 * setValue / analogReadOversampling(RX_VOLTAGE_2_PIN, CALIBRATION_OVERSAMPLINGRATE);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibration Done");
          break;
        case DRIVE_VOLTAGE_CALIBRATE_VOLTAGE:
          set.rxVoltage2Scale = 0.01 * setValue / analogReadOversampling(DRIVE_VOLTAGE_PIN, CALIBRATION_OVERSAMPLINGRATE);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibration Done");
          break;
        case DRIVE_CURRENT_SET_SCALE:
          set.driveCurrentScale = 0.01 * setValue * VCC / 1023;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Done");
          break;
        case PITOT_PRESSURE_SET_SCALE:
          set.pitotPressureScale = setValue * VCC / 1023;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Done");
          break;
        case AIRSPEED_SET_RATIO:
          set.airspeedRatio = 0.001 * setValue;
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Done");
          break;
      }
      menuLevel = MAIN;
      return;
    }
    return;
  }
  
  if (menuLevel == INFO) {
    if (key == LEFT) {
      menuLevel = MAIN;
      setJetiboxText();
      return;
    }
    if (key == RIGHT) {
      switch (menu) {
        case DRIVE_CURRENT_CALIBRATE_OFFSET:
          set.driveCurrentOffset = analogReadOversampling(DRIVE_CURRENT_PIN, CALIBRATION_OVERSAMPLINGRATE);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibration Done");
          break;
        case PITOT_PRESSURE_CALIBRATE_OFFSET:
          set.pitotPressureOffset = analogReadOversampling(PITOT_PRESSURE_PIN, CALIBRATION_OVERSAMPLINGRATE);
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibration Done");
          break;
      }
      menuLevel = MAIN;
      return;
    }
    return;
  }
}

void menuLevelSet(float value, float valueMin, float valueMax, int decimalDigits) {
  setDecimalFactor = 1;
  for (int i = 0; i < decimalDigits; i++) setDecimalFactor *= 10;
  
  setValue = value * setDecimalFactor;
  setValueMin = valueMin * setDecimalFactor;
  setValueMax = valueMax * setDecimalFactor;
  if (setValue < setValueMin) setValue = setValueMin;    
  if (setValue > setValueMax) setValue = setValueMax;
  
  setIncrement = 1;
  for (int i = 0; i < int(log10(setValue)); i++) setIncrement *= 10;
  setIncrementMax = 1;
  for (int i = 0; i < int(log10(setValueMax)); i++) setIncrementMax *= 10;

  menuLevel = SET;
  setJetiboxText();
}

void setJetiboxText() {
  if (menuLevel == MAIN) {
    switch (menu) {
      case SETTINGS_SAVE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "SETTINGS");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Save");
        break;
      case ALTITUDE_RESET:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "ALTITUDE");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Reset");
        break;
      case VARIO_SET_TEC:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "VARIO");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set TEC");
        break;
      case RX_VOLTAGE_1_CALIBRATE_VOLTAGE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "RX VOLTAGE 1");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibrate Volt.");
        break;
      case RX_VOLTAGE_2_CALIBRATE_VOLTAGE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "RX VOLTAGE 2");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibrate Volt.");
        break;
      case DRIVE_VOLTAGE_CALIBRATE_VOLTAGE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "DRIVE VOLTAGE");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibrate Volt.");
        break;
      case DRIVE_CURRENT_CALIBRATE_OFFSET:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "DRIVE CURRENT");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibrate Offset");
        break;
      case DRIVE_CURRENT_SET_SCALE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "DRIVE CURRENT");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Scale");
        break;
      case PITOT_PRESSURE_CALIBRATE_OFFSET:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "PITOT PRESSURE");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Calibrate Offset");
        break;
      case PITOT_PRESSURE_SET_SCALE:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "PITOT PRESSURE");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Scale");
        break;
      case AIRSPEED_SET_RATIO:
        jetiEx.SetJetiboxText(JetiExProtocol::LINE1, "AIRSPEED");
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "Set Ratio");
        break;
    }
    return;
  }
  
  if (menuLevel == SET) {
    char _buffer[17];
    switch (menu) {
      case VARIO_SET_TEC:
        if (setValue > 0) {
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "TEC On");
        } else
          jetiEx.SetJetiboxText(JetiExProtocol::LINE2, "TEC Off");
        break;
      case RX_VOLTAGE_1_CALIBRATE_VOLTAGE:
        sprintf(_buffer, "%1d.%03dV     %1d.%03d", int(setValue / setDecimalFactor), int(setValue % setDecimalFactor), int(setIncrement / setDecimalFactor), int(setIncrement % setDecimalFactor));
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
      case RX_VOLTAGE_2_CALIBRATE_VOLTAGE:
        sprintf(_buffer, "%1d.%03dV     %1d.%03d", int(setValue / setDecimalFactor), int(setValue % setDecimalFactor), int(setIncrement / setDecimalFactor), int(setIncrement % setDecimalFactor));
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
      case DRIVE_VOLTAGE_CALIBRATE_VOLTAGE:
        sprintf(_buffer, "%2d.%02dV     %2d.%02d", int(setValue / setDecimalFactor), int(setValue % setDecimalFactor), int(setIncrement / setDecimalFactor), int(setIncrement % setDecimalFactor));
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
      case DRIVE_CURRENT_SET_SCALE:
        sprintf(_buffer, "%2d.%02dA/V   %2d.%02d", int(setValue / setDecimalFactor), int(setValue % setDecimalFactor), int(setIncrement / setDecimalFactor), int(setIncrement % setDecimalFactor));
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
      case PITOT_PRESSURE_SET_SCALE:
        sprintf(_buffer, "%4dPa/V    %4d", setValue, setIncrement);
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
      case AIRSPEED_SET_RATIO:
        sprintf(_buffer, "%1d.%03d      %1d.%03d", int(setValue / setDecimalFactor), int(setValue % setDecimalFactor), int(setIncrement / setDecimalFactor), int(setIncrement % setDecimalFactor));
        jetiEx.SetJetiboxText(JetiExProtocol::LINE2, _buffer);
        break;
    }
    return;
  }
}

